package btree;

public class BTreeFac implements IBTreeFac {
  
  public static final BTreeFac Singleton = new BTreeFac();
  private BTreeFac() {}
  
  private IMTBTree _MTBTree = new IMTBTree() {
    public Object execute(IBTreeAlgo algo, Object... params) {
      return algo.mtCase(this, params);
    }
    public String toString() {
      return (String) execute(btree.visitor.ToString.Singleton);
    }
    
  };
  
  private class NEBTree implements INEBTree {
    private Object[] _data;
    private IBTree[] _children;
    
    NEBTree(Object[] newData, IBTree[] newChildren) {
      _data = new Object[newData.length];
      _children = new IBTree[newData.length+1];
      for(int i=0; i<newData.length; i++) {
        _data[i] = newData[i];
        _children[i] = newChildren[i];
      }
      _children[newData.length] = newChildren[newData.length];
    }
    
    public Object[] getData() {
      return _data.clone();  // return a copy of the data array to prevent mutation.
    }
    public IBTree[] getChildren(){
      return _children.clone(); // return a copy of the child array to prevent mutation. 
    }
    
    public Object execute(IBTreeAlgo algo, Object... params) {
      return algo.neCase(this, params);
    }
    public String toString() {
      return (String) execute(btree.visitor.ToString.Singleton);
    }
    
  }
  
  public IMTBTree makeMTBTree() {
    return _MTBTree;
  }
  
  public INEBTree makeNEBTree(Object[] newData, IBTree[] newChildren) {
    if(newData.length==0 || (newData.length != newChildren.length-1)) {
      throw new IllegalArgumentException("Wrong number of data elements ("
                                           +newData.length
                                           +") vs. number of children ("
                                           +newChildren.length+")!");
    }
    else {
      // anArray.clone() returns a copy of the array.
      // The copy is performed to isolate the data structure and make it
      // immutable because arrays are fundamentally mutable.
      return new NEBTree(newData.clone(), newChildren.clone());
    }
  }
  
  
}