package patient;
import java.util.*;

/**
 * This interface represents an abstract patient, which can be sane or insane. 
 * @dependency patient.IPatientVisitor uses
 */
public interface IPatient extends Observer
{
 /**
  * Call by a central dispatcher to update the patient's sanity condition.  
  * @param o The central dispatcher (Observable)
  * @param param Ignored
  */
 public void update(Observable o, Object param);
 /**
  * Executes a Visitor design pattern algorithm on the patient.
  * @param algo The algorithm to be run.
  * @param param An input paramter for the algorithm.
  * @return The return value of the algorithm.
  */
 public Object execute(IPatientVisitor algo, Object... param);

}