package patient.test;

import patient.*;
import rac.*;
import junit.framework.TestCase;


public class TAMUFactory_Test extends TestCase {
  
  IRACFactory racFac = new TAMUFactory();
  IPatient nullPatient = NullPatient.Singleton;
  
  public void test_TAMU() {
    IRAContainer tamu = racFac.makeRAC();
    assertTrue("Empty initial TAMU", tamu.isEmpty());                                   
    assertEquals("Empty TAMU, peek", nullPatient, tamu.peek()); 
    assertEquals("Empty TAMU, get", nullPatient, tamu.get()); 
    
    final IPatient sane1 = new Patient(true);
    tamu.put(sane1);
    assertEquals("1 sane patient, peek", sane1, tamu.peek());
    assertEquals("1 sane patient, get", sane1, tamu.get());
    assertTrue("Now empty TAMU", tamu.isEmpty());                                   
    
    tamu.put(new Patient(false));
    assertEquals("1 insane patient, peek", nullPatient, tamu.peek());
    assertEquals("1 insane patient, get", nullPatient, tamu.get());
    assertFalse("{insane}, isEmpty", tamu.isEmpty());
    
    tamu.put(sane1);
    assertEquals("{sane, insane}, peek", sane1, tamu.peek());
    assertEquals("{sane, insane}, get", sane1, tamu.get());
    assertFalse("{insane}, isEmpty", tamu.isEmpty());
    assertEquals("1 insane patient, peek", nullPatient, tamu.peek());
    assertEquals("1 insane patient, get", nullPatient, tamu.get());
    assertFalse("{insane}, isEmpty", tamu.isEmpty());
    
    tamu.clear();
    tamu.put(sane1);
    tamu.put(new Patient(false));
    tamu.put(new Patient(false));
    assertEquals("{insane, insane, sane}, peek", sane1, tamu.peek());
    assertEquals("{insane, insane, sane}, get", sane1, tamu.get());
    assertEquals("{insane, insane}, peek", nullPatient, tamu.peek());
    assertEquals("{insane, insane}, get", nullPatient, tamu.get());
    assertFalse("{insane, insane}, isEmpty", tamu.isEmpty());
    
    final IPatient sane2 = new Patient(true);
    
    tamu.put(sane2);
    tamu.put(new Patient(false));
    tamu.put(sane1);
    tamu.put(new Patient(false));
    tamu.put(new Patient(false));
    
    // Test object that "equals" either sane1 or sane2
    // Allows one to test against either object being returned.
    Object testObj = new Object() {
      public boolean equals(Object o) {
        return sane1.equals(o) || sane2.equals(o);
      }
    };
    
    assertEquals("{insane, insane, sane, insane, sane, insane, insane}, peek", testObj, tamu.peek());
    assertEquals("{insane, insane, sane, insane, sane, insane, insane}, get", testObj, tamu.get());
    assertEquals("{insane, insane, sane, insane, sane, insane, insane}, peek", testObj, tamu.peek());
    assertEquals("{insane, insane, sane, insane, sane, insane, insane}, get", testObj, tamu.get());
    assertEquals("{insane, insane, insane, insane, insane}, peek", nullPatient, tamu.peek());
    assertEquals("{insane, insane, insane, insane, insane}, get", nullPatient, tamu.get());
    assertFalse("{insane, insane, insane, insane, insane}, isEmpty", tamu.isEmpty());
    
  }
  
}
