package rac.test;
import rac.*;
import junit.framework.TestCase;

public class Queue_Test extends TestCase {
  
  IRACFactory racFac1 = new LRSQueueFactory();
  IRACFactory racFac2 = new LRSQueue2Factory();
  
  
  public void test_empty_queue() {
    IRAContainer rac = racFac1.makeRAC();
    
    try {
      Object x = rac.peek();
      fail("Empty RAC peek returned: "+ x);
    }
    catch(IllegalStateException e) {
      System.out.println("Empty RAC peek correctly threw exception: "+e);
    }
    catch(Exception e) {
      fail("Empty RAC peek, wrong exception thrown: "+e);
    }
    
    try {
      Object x = rac.get();
      fail("Empty RAC get returned: "+ x);
    }
    catch(IllegalStateException e) {
      System.out.println("Empty RAC get correctly threw exception: "+e);
    }
    catch(Exception e) {
      fail("Empty RAC get, wrong exception thrown: "+e);
    }
      
  }
  
  public void test_nonEmpty_queue() {
    IRAContainer rac = racFac1.makeRAC();
    
    rac.put("a");
     
    assertEquals("1 element rac peek", "a", rac.peek());
    
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
    
    rac.put("b");
    rac.put("a");
    assertEquals("2 element rac peek", "b", rac.peek());
    assertEquals("2 element rac get", "b", rac.get());
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
    
    rac.put("c");
    rac.put("b");
    rac.put("a");
    assertEquals("3 element rac peek", "c", rac.peek());
    assertEquals("3 element rac get", "c", rac.get());
    assertEquals("2 element rac get", "b", rac.get());
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
  }
    
  public void test_empty_queue2() {
    IRAContainer rac = racFac2.makeRAC();
    
    try {
      Object x = rac.peek();
      fail("Empty RAC peek returned: "+ x);
    }
    catch(IllegalStateException e) {
      System.out.println("Empty RAC peek correctly threw exception: "+e);
    }
    catch(Exception e) {
      fail("Empty RAC peek, wrong exception thrown: "+e);
    }
    
    try {
      Object x = rac.get();
      fail("Empty RAC get returned: "+ x);
    }
    catch(IllegalStateException e) {
      System.out.println("Empty RAC get correctly threw exception: "+e);
    }
    catch(Exception e) {
      fail("Empty RAC get, wrong exception thrown: "+e);
    }
      
  }


  public void test_nonEmpty_queue2() {
    IRAContainer rac = racFac2.makeRAC();
    
    rac.put("a");
     
    assertEquals("1 element rac peek", "a", rac.peek());
    
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
    
    rac.put("b");
    rac.put("a");
    assertEquals("2 element rac peek", "b", rac.peek());
    assertEquals("2 element rac get", "b", rac.get());
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
    
    rac.put("c");
    rac.put("b");
    rac.put("a");
    assertEquals("3 element rac peek", "c", rac.peek());
    assertEquals("3 element rac get", "c", rac.get());
    assertEquals("2 element rac get", "b", rac.get());
    assertEquals("1 element rac get", "a", rac.get());
    assertTrue("Empty rac", rac.isEmpty());
  }
  public void test_queue_vs_queue2() {
    IRAContainer rac1 = racFac1.makeRAC();
    IRAContainer rac2 = racFac2.makeRAC();
    
    int nLoops = 1000;
    long t0 = System.currentTimeMillis();
    for(int i=0;i<nLoops;i++) {
      rac1.put(i);
    }
    long deltaT1 = System.currentTimeMillis()-t0;
    
    t0 = System.currentTimeMillis();
    for(int i=0;i<nLoops;i++) {
      rac2.put(i);
    }
    long deltaT2 = System.currentTimeMillis()-t0;
    
    assertTrue("rac2.put should be faster than rac1.put", 
               deltaT2 < deltaT1);

    t0 = System.currentTimeMillis();
    for(int i=0;i<nLoops;i++) {
      rac1.get();
    }
    deltaT1 = System.currentTimeMillis()-t0;
    
    t0 = System.currentTimeMillis();
    for(int i=0;i<nLoops;i++) {
      rac2.get();
    }
    deltaT2 = System.currentTimeMillis()-t0;
    assertTrue("rac.get should be faster than rac2.gat", 
               deltaT1 < deltaT2);
  
  }  
}
